#include <string.h>

#include <dos/dosextens.h>
#include <exec/execbase.h>
#include <proto/dos.h>
#include <proto/exec.h>

#include "Main.h"

struct programlibs
{
	CONST_STRPTR  name;
	size_t        version;
	APTR         *base;
};

struct ExecBase      *SysBase;
struct DosLibrary    *DOSBase;
struct Library       *LocaleBase;
struct IntuitionBase *IntuitionBase;
struct Library       *MUIMasterBase;
struct Library       *IconBase;

STATIC const struct programlibs libs[] =
{
	{ "icon.library"         ,  0, (APTR)&IconBase },
	{ "intuition.library"    , 36, (APTR)&IntuitionBase },
	{ "muimaster.library"    , 13, (APTR)&MUIMasterBase },
	{ "locale.library"       , 38, (APTR)&LocaleBase },
};

int main(void)
{
	struct WBStartup *wbmsg;
	struct Process *GUIThread;
	int result;

	wbmsg     = NULL;
	SysBase   = (APTR)(*(size_t *)4);
	GUIThread = (struct Process *)SysBase->ThisTask;
	result    = RETURN_FAIL;

	if (GUIThread->pr_CLI == 0)
	{
		WaitPort(&GUIThread->pr_MsgPort);
		wbmsg	= (struct WBStartup *)GetMsg(&GUIThread->pr_MsgPort);
	}

	DOSBase = (APTR)OpenLibrary("dos.library", 36);

	if (DOSBase)
	{
		size_t i;
		IPTR base = TRUE;

		for (i = 0; base && i < sizeof(libs) / sizeof(struct programlibs); i++)
		{
			base = (IPTR)OpenLibrary(libs[i].name, libs[i].version);
			*libs[i].base = (APTR)base;
		}

		if (base)
		{
			extern int start(struct WBStartup *wbmsg, IPTR *arglist);

			if (!wbmsg)
			{
				struct RDArgs *args;
				IPTR arglist[ARG_COUNT];

				bzero(arglist, sizeof(arglist));
				args = ReadArgs(CMDARG, arglist, NULL);

				if (args)
				{
					start(NULL, arglist);
					FreeArgs(args);
				}
				else
				{
					PrintFault(IoErr(), "ERROR");
				}
			}
			else
			{
				start(wbmsg, NULL);
			}

			CloseLibrary(IconBase);
			result = RETURN_OK;
		}

		for (i = 0; i < sizeof(libs) / sizeof(struct programlibs); i++)
			CloseLibrary(*libs[i].base);
	}

	if (wbmsg)
	{
		Forbid();
		ReplyMsg((struct Message *)wbmsg);
	}

	return result;
}

#if defined(__PPC__)
CONST ULONG __abox__ = 1;
#endif
